@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,8 @@
 #include <gtk/gtktextview.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
+#include <gtk/gtkcheckmenuitem.h>
+#include <gtk/gtkstatusbar.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
@@ -56,6 +58,7 @@
 #include "prefs_common.h"
 #include "prefs_account.h"
 #include "prefs_filter.h"
+#include "filter.h"
 #include "gtkutils.h"
 #include "utils.h"
 #include "rfc2015.h"
@@ -64,7 +67,10 @@ static GList *messageview_list = NULL;
 
 static void messageview_change_view_type(MessageView		*messageview,
 					 MessageType		 type);
-static void messageview_destroy_cb	(GtkWidget		*widget,
+static void messageview_set_menu_state	(MessageView		*messageview);
+
+static gint messageview_delete_cb	(GtkWidget		*widget,
+					 GdkEventAny		*event,
 					 MessageView		*messageview);
 static void messageview_size_allocate_cb(GtkWidget		*widget,
 					 GtkAllocation		*allocation);
@@ -165,6 +171,8 @@ static GtkItemFactoryEntry msgview_entries[] =
 	 ENC_ACTION(C_ISO_8859_1)},
 	{N_("/_View/Character _encoding/Western European (ISO-8859-15)"),
 	 ENC_ACTION(C_ISO_8859_15)},
+	{N_("/_View/Character _encoding/Western European (Windows-1252)"),
+	 ENC_ACTION(C_WINDOWS_1252)},
 	ENC_SEPARATOR,
 	{N_("/_View/Character _encoding/Central European (ISO-8859-_2)"),
 	 ENC_ACTION(C_ISO_8859_2)},
@@ -173,10 +181,22 @@ static GtkItemFactoryEntry msgview_entries[] =
 	 ENC_ACTION(C_ISO_8859_13)},
 	{N_("/_View/Character _encoding/Baltic (ISO-8859-_4)"),
 	 ENC_ACTION(C_ISO_8859_4)},
+	{N_("/_View/Character _encoding/Baltic (Windows-1257)"),
+	 ENC_ACTION(C_WINDOWS_1257)},
 	ENC_SEPARATOR,
 	{N_("/_View/Character _encoding/Greek (ISO-8859-_7)"),
 	 ENC_ACTION(C_ISO_8859_7)},
 	ENC_SEPARATOR,
+	{N_("/_View/Character _encoding/Arabic (ISO-8859-_6)"),
+	 ENC_ACTION(C_ISO_8859_6)},
+	{N_("/_View/Character _encoding/Arabic (Windows-1256)"),
+	 ENC_ACTION(C_CP1256)},
+	ENC_SEPARATOR,
+	{N_("/_View/Character _encoding/Hebrew (ISO-8859-_8)"),
+	 ENC_ACTION(C_ISO_8859_8)},
+	{N_("/_View/Character _encoding/Hebrew (Windows-1255)"),
+	 ENC_ACTION(C_CP1255)},
+	ENC_SEPARATOR,
 	{N_("/_View/Character _encoding/Turkish (ISO-8859-_9)"),
 	 ENC_ACTION(C_ISO_8859_9)},
 	ENC_SEPARATOR,
@@ -224,7 +244,8 @@ static GtkItemFactoryEntry msgview_entries[] =
 
 	{N_("/_View/---"),		NULL, NULL, 0, "<Separator>"},
 	{N_("/_View/Mess_age source"),	NULL, view_source_cb, 0, NULL},
-	{N_("/_View/Show all _header"),	NULL, show_all_header_cb, 0, "<ToggleItem>"},
+	{N_("/_View/Show all _headers"),
+					NULL, show_all_header_cb, 0, "<ToggleItem>"},
 
 	{N_("/_Message"),		NULL, NULL, 0, "<Branch>"},
 	{N_("/_Message/Compose _new message"),
@@ -253,15 +274,17 @@ static GtkItemFactoryEntry msgview_entries[] =
 	{N_("/_Tools/_Create filter rule"),
 					NULL, NULL, 0, "<Branch>"},
 	{N_("/_Tools/_Create filter rule/_Automatically"),
-					NULL, create_filter_cb, FILTER_BY_AUTO, NULL},
+					NULL, create_filter_cb, FLT_BY_AUTO, NULL},
 	{N_("/_Tools/_Create filter rule/by _From"),
-					NULL, create_filter_cb, FILTER_BY_FROM, NULL},
+					NULL, create_filter_cb, FLT_BY_FROM, NULL},
 	{N_("/_Tools/_Create filter rule/by _To"),
-					NULL, create_filter_cb, FILTER_BY_TO, NULL},
+					NULL, create_filter_cb, FLT_BY_TO, NULL},
 	{N_("/_Tools/_Create filter rule/by _Subject"),
-					NULL, create_filter_cb, FILTER_BY_SUBJECT, NULL},
+					NULL, create_filter_cb, FLT_BY_SUBJECT, NULL},
+#ifndef G_OS_WIN32
 	{N_("/_Tools/---"),		NULL, NULL, 0, "<Separator>"},
 	{N_("/_Tools/Actio_ns"),	NULL, NULL, 0, "<Branch>"},
+#endif
 
 	{N_("/_Help"),			NULL, NULL, 0, "<Branch>"},
 	{N_("/_Help/_About"),		NULL, about_cb, 0, NULL}
@@ -357,7 +380,7 @@ MessageView *messageview_create_with_new_window(void)
 	guint n_menu_entries;
 
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_title(GTK_WINDOW(window), _("Sylpheed - Message View"));
+	gtk_window_set_title(GTK_WINDOW(window), _("Message View - Sylpheed"));
 	gtk_window_set_wmclass(GTK_WINDOW(window), "message_view", "Sylpheed");
 	gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, FALSE);
 	gtk_widget_set_size_request(window, prefs_common.msgwin_width,
@@ -371,8 +394,8 @@ MessageView *messageview_create_with_new_window(void)
 	g_signal_connect(G_OBJECT(window), "size_allocate",
 			 G_CALLBACK(messageview_size_allocate_cb),
 			 msgview);
-	g_signal_connect(G_OBJECT(window), "destroy",
-			 G_CALLBACK(messageview_destroy_cb), msgview);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(messageview_delete_cb), msgview);
 	g_signal_connect(G_OBJECT(window), "key_press_event",
 			 G_CALLBACK(key_pressed), msgview);
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
@@ -380,7 +403,6 @@ MessageView *messageview_create_with_new_window(void)
 	n_menu_entries = sizeof(msgview_entries) / sizeof (msgview_entries[0]);
 	menubar = menubar_create(window, msgview_entries, n_menu_entries,
 				 "<MessageView>", msgview);
-#warning FIXME_GTK2
 #if 0
 	menu_factory_copy_rc("<Main>", "<MessageView>");
 #endif
@@ -406,12 +428,16 @@ MessageView *messageview_create_with_new_window(void)
 	msgview->window = window;
 	msgview->window_vbox = window_vbox;
 	msgview->body_vbox = body_vbox;
+	msgview->menubar = menubar;
+	msgview->menu_locked = FALSE;
 	msgview->visible = TRUE;
 
 	messageview_init(msgview);
 
 	ifactory = gtk_item_factory_from_widget(menubar);
+#ifndef G_OS_WIN32
 	action_update_msgview_menu(ifactory, msgview);
+#endif
 
 	messageview_list = g_list_append(messageview_list, msgview);
 
@@ -461,6 +487,14 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 		if (!messageview->msginfo)
 			messageview->msginfo = procmsg_msginfo_copy(msginfo);
 	}
+
+	if (messageview->window && msginfo->subject) {
+		gchar *title;
+
+		title = g_strconcat(msginfo->subject, " - Sylpheed", NULL);
+		gtk_window_set_title(GTK_WINDOW(messageview->window), title);
+		g_free(title);
+	}
 	headerview_show(messageview->headerview, messageview->msginfo);
 
 	textview_set_all_headers(messageview->textview, all_headers);
@@ -476,6 +510,9 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 		procmime_mimeinfo_free_all(mimeinfo);
 	}
 
+	if (messageview->new_window)
+		messageview_set_menu_state(messageview);
+
 	g_free(file);
 
 	return 0;
@@ -505,6 +542,20 @@ static void messageview_change_view_type(MessageView *messageview,
 	messageview->type = type;
 }
 
+static void messageview_set_menu_state(MessageView *messageview)
+{
+	GtkItemFactory *ifactory;
+	GtkWidget *menuitem;
+
+	messageview->menu_locked = TRUE;
+	ifactory = gtk_item_factory_from_widget(messageview->menubar);
+	menuitem = gtk_item_factory_get_widget
+		(ifactory, "/View/Show all headers");
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem),
+				       messageview->textview->show_all_headers);
+	messageview->menu_locked = FALSE;
+}
+
 void messageview_clear(MessageView *messageview)
 {
 	procmsg_msginfo_free(messageview->msginfo);
@@ -528,6 +579,9 @@ void messageview_destroy(MessageView *messageview)
 
 	procmsg_msginfo_free(messageview->msginfo);
 
+	if (messageview->window)
+		gtk_widget_destroy(messageview->window);
+
 	g_free(messageview);
 
 	gtk_widget_unref(textview);
@@ -653,9 +707,11 @@ void messageview_save_as(MessageView *messageview)
 	g_free(dest);
 }
 
-static void messageview_destroy_cb(GtkWidget *widget, MessageView *messageview)
+static gint messageview_delete_cb(GtkWidget *widget, GdkEventAny *event,
+				  MessageView *messageview)
 {
 	messageview_destroy(messageview);
+	return TRUE;
 }
 
 static void messageview_size_allocate_cb(GtkWidget *widget,
@@ -677,8 +733,10 @@ static void messageview_switch_page_cb(GtkNotebook *notebook,
 static gboolean key_pressed(GtkWidget *widget, GdkEventKey *event,
 			    MessageView *messageview)
 {
-	if (event && event->keyval == GDK_Escape && messageview->window)
-		gtk_widget_destroy(messageview->window);
+	if (event && event->keyval == GDK_Escape && messageview->window) {
+		messageview_destroy(messageview);
+		return TRUE;
+	}
 	return FALSE;
 }
 
@@ -691,32 +749,38 @@ static void save_as_cb(gpointer data, guint action, GtkWidget *widget)
 static void print_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	MessageView *messageview = (MessageView *)data;
-	gchar *cmdline;
-	gchar *p;
+	const gchar *cmdline;
+	gchar *msg;
 
 	if (!messageview->msginfo) return;
 
-	cmdline = input_dialog(_("Print"),
-			       _("Enter the print command line:\n"
-				 "(`%s' will be replaced with file name)"),
-			       prefs_common.print_cmd);
-	if (!cmdline) return;
-	if (!(p = strchr(cmdline, '%')) || *(p + 1) != 's' ||
-	    strchr(p + 2, '%')) {
+	cmdline = prefs_common.print_cmd;
+
+	msg = g_strconcat
+		(_("The message will be printed with the following command:"),
+		 "\n\n", cmdline ? cmdline : _("(Default print command)"),
+		 NULL);
+	if (alertpanel(_("Print"), msg, GTK_STOCK_OK, GTK_STOCK_CANCEL, NULL)
+	    != G_ALERTDEFAULT) {
+		g_free(msg);
+		return;
+	}
+	g_free(msg);
+
+	if (cmdline && str_find_format_times(cmdline, 's') != 1) {
 		alertpanel_error(_("Print command line is invalid:\n`%s'"),
 				 cmdline);
-		g_free(cmdline);
 		return;
 	}
 
-	procmsg_print_message(messageview->msginfo, cmdline);
-	g_free(cmdline);
+	procmsg_print_message(messageview->msginfo, cmdline,
+			      messageview->textview->show_all_headers);
 }
 
 static void close_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	MessageView *messageview = (MessageView *)data;
-	gtk_widget_destroy(messageview->window);
+	messageview_destroy(messageview);
 }
 
 static void copy_cb(gpointer data, guint action, GtkWidget *widget)
@@ -768,6 +832,8 @@ static void show_all_header_cb(gpointer data, guint action, GtkWidget *widget)
 	MsgInfo *msginfo = messageview->msginfo;
 
 	if (!msginfo) return;
+	if (messageview->menu_locked) return;
+
 	messageview->msginfo = NULL;
 	messageview_show(messageview, msginfo,
 			 GTK_CHECK_MENU_ITEM(widget)->active);
@@ -885,8 +951,8 @@ static void create_filter_cb(gpointer data, guint action, GtkWidget *widget)
 
 	if (!messageview->msginfo) return;
 
-	procmsg_get_filter_keyword(messageview->msginfo, &header, &key,
-				   (PrefsFilterType)action);
+	filter_get_keyword_from_msg(messageview->msginfo, &header, &key,
+				    (FilterCreateType)action);
 	prefs_filter_open(messageview->msginfo, header);
 
 	g_free(header);
