@@ -35,6 +35,7 @@
 
 #include <gpgme.h>
 
+#include "procmsg.h"
 #include "procmime.h"
 #include "procheader.h"
 #include "base64.h"
@@ -50,6 +51,8 @@
 
 #define DIM(v)     (sizeof(v) / sizeof((v)[0]))
 
+static gboolean gpg_available = TRUE;
+
 static gchar *content_names[] = {
 	"Content-Type",
 	"Content-Disposition",
@@ -105,8 +108,12 @@ static void dump_part(MimeInfo *mimeinfo, FILE *fp)
 
 void rfc2015_disable_all(void)
 {
-	/* FIXME: set a flag, so that we don't bother the user with failed
-	 * gpgme messages */
+	gpg_available = FALSE;
+}
+
+gboolean rfc2015_is_available(void)
+{
+	return gpg_available;
 }
 
 void rfc2015_secure_remove(const gchar *fname)
@@ -114,7 +121,7 @@ void rfc2015_secure_remove(const gchar *fname)
 	if (!fname)
 		return;
 	/* fixme: overwrite the file first */
-	remove(fname);
+	g_remove(fname);
 }
 
 static void sig_status_for_key(GString *str, gpgme_ctx_t ctx,
@@ -122,9 +129,13 @@ static void sig_status_for_key(GString *str, gpgme_ctx_t ctx,
 {
 	gpgme_key_t key;
 	gpgme_user_id_t user;
+	gpgme_error_t err;
 
-	gpgme_get_key(ctx, sig->fpr, &key, 0);
-	if (key == NULL || key->uids->uid == NULL) {
+	err = gpgme_get_key(ctx, sig->fpr, &key, 0);
+	if (err || key == NULL || key->uids->uid == NULL) {
+		if (err)
+			debug_print("gpgme_get_key failed: %s\n",
+				    gpgme_strerror(err));
 		g_string_sprintfa(str, "%s\n",
 		                  gpgmegtk_sig_status_to_string (sig, FALSE));
 		if ((sig->fpr != NULL) && (*(sig->fpr) != '\0'))
@@ -165,7 +176,10 @@ static gchar *sig_status_full(gpgme_ctx_t ctx, gpgme_verify_result_t result)
 			ctime_val = localtime(&created);
 			my_strftime(ctime_str, sizeof (ctime_str), "%c",
 			            ctime_val);
-			ctime_str_utf8 = conv_localetodisp(ctime_str, NULL);
+			ctime_str_utf8 = g_locale_to_utf8(ctime_str, -1,
+							  NULL, NULL, NULL);
+			if (!ctime_str_utf8)
+				ctime_str_utf8 = g_strdup(ctime_str);
 			g_string_sprintfa(str, _("Signature made at %s\n"),
 			                  ctime_str_utf8);
 			g_free(ctime_str_utf8);
@@ -201,6 +215,11 @@ static void check_signature(MimeInfo *mimeinfo, MimeInfo *partinfo, FILE *fp)
 		goto leave;
 	}
 
+	if (rfc2015_is_pkcs7_signature_part(partinfo)) {
+		debug_print("pkcs7 signature detected\n");
+		gpgme_set_protocol(ctx, GPGME_PROTOCOL_CMS);
+	}
+
 	/* don't include the last empty line.
 	   It does not belong to the signed text */
 	if (mimeinfo->children->size > 0) {
@@ -232,14 +251,14 @@ static void check_signature(MimeInfo *mimeinfo, MimeInfo *partinfo, FILE *fp)
 		goto leave;
 	}
 	if (canonicalize_file_replace(tmp_file) < 0) {
-		unlink(tmp_file);
+		g_unlink(tmp_file);
 		g_free(tmp_file);
 		goto leave;
 	}
 
 	err = gpgme_data_new_from_file(&text, tmp_file, 1);
 
-	unlink(tmp_file);
+	g_unlink(tmp_file);
 	g_free(tmp_file);
 
 	if (!err)
@@ -252,6 +271,17 @@ static void check_signature(MimeInfo *mimeinfo, MimeInfo *partinfo, FILE *fp)
 		goto leave;
 	}
 
+#if 0
+	if (partinfo->encoding_type == ENC_BASE64) {
+		err = gpgme_data_set_encoding(sig, GPGME_DATA_ENCODING_BASE64);
+		if (err) {
+			debug_print("gpgme_data_set_encoding failed: %s\n",
+			            gpgme_strerror (err));
+			goto leave;
+		}
+	}
+#endif
+
 	err = gpgme_op_verify(ctx, sig, text, NULL);
 	if (err)  {
 		debug_print("gpgme_op_verify failed: %s\n",
@@ -305,7 +335,7 @@ static gchar *copy_gpgmedata_to_temp(GpgmeData data, guint *length)
 	tmp = g_strdup_printf("%s%cgpgtmp.%08x",
 	                      get_mime_tmp_dir(), G_DIR_SEPARATOR, ++id);
 
-	if ((fp = fopen(tmp, "wb")) == NULL) {
+	if ((fp = g_fopen(tmp, "wb")) == NULL) {
 		FILE_OP_ERROR(tmp, "fopen");
 		g_free(tmp);
 		return NULL;
@@ -411,8 +441,7 @@ MimeInfo **rfc2015_find_signature(MimeInfo *mimeinfo)
 	/* check that we have at least 2 parts of the correct type */
 	for (partinfo = mimeinfo->children;
 	        partinfo != NULL; partinfo = partinfo->next) {
-		if (++n > 1  && !g_ascii_strcasecmp(partinfo->content_type,
-		                                    "application/pgp-signature"))
+		if (++n > 1  && rfc2015_is_signature_part(partinfo))
 			break;
 	}
 
@@ -450,6 +479,34 @@ void rfc2015_check_signature(MimeInfo *mimeinfo, FILE *fp)
 	g_free(signedinfo);
 }
 
+gboolean rfc2015_is_pgp_signature_part(MimeInfo *mimeinfo)
+{
+	if (!mimeinfo || !mimeinfo->content_type)
+		return FALSE;
+
+	return !g_ascii_strcasecmp(mimeinfo->content_type,
+				   "application/pgp-signature");
+}
+
+gboolean rfc2015_is_pkcs7_signature_part(MimeInfo *mimeinfo)
+{
+	const gchar *c_type;
+
+	if (!mimeinfo || !mimeinfo->content_type)
+		return FALSE;
+
+	c_type = mimeinfo->content_type;
+
+	return (!g_ascii_strcasecmp(c_type, "application/pkcs7-signature") ||
+		!g_ascii_strcasecmp(c_type, "application/x-pkcs7-signature"));
+}
+
+gboolean rfc2015_is_signature_part(MimeInfo *mimeinfo)
+{
+	return (rfc2015_is_pgp_signature_part(mimeinfo) ||
+		rfc2015_is_pkcs7_signature_part(mimeinfo));
+}
+
 gint rfc2015_is_encrypted(MimeInfo *mimeinfo)
 {
 	if (!mimeinfo || mimeinfo->mime_type != MIME_MULTIPART)
@@ -466,7 +523,7 @@ gboolean rfc2015_msg_is_encrypted(const gchar *file)
 	MimeInfo *mimeinfo;
 	gint ret;
 
-	if ((fp = fopen(file, "rb")) == NULL)
+	if ((fp = g_fopen(file, "rb")) == NULL)
 		return FALSE;
 
 	mimeinfo = procmime_scan_mime_header(fp);
@@ -586,7 +643,7 @@ void rfc2015_decrypt_message(MsgInfo *msginfo, MimeInfo *mimeinfo, FILE *fp)
 	fname = g_strdup_printf("%s%cplaintext.%08x",
 	                        get_mime_tmp_dir(), G_DIR_SEPARATOR, ++id);
 
-	if ((dstfp = fopen(fname, "wb")) == NULL) {
+	if ((dstfp = g_fopen(fname, "wb")) == NULL) {
 		FILE_OP_ERROR(fname, "fopen");
 		g_free(fname);
 		DECRYPTION_ABORT();
@@ -637,6 +694,55 @@ void rfc2015_decrypt_message(MsgInfo *msginfo, MimeInfo *mimeinfo, FILE *fp)
 
 #undef DECRYPTION_ABORT
 
+FILE *rfc2015_open_message_decrypted(MsgInfo *msginfo, MimeInfo **mimeinfo)
+{
+	FILE *fp;
+	MimeInfo *mimeinfo_;
+	glong fpos;
+
+	g_return_val_if_fail(msginfo != NULL, NULL);
+
+	if (mimeinfo) *mimeinfo = NULL;
+
+	if ((fp = procmsg_open_message(msginfo)) == NULL) return NULL;
+
+	mimeinfo_ = procmime_scan_mime_header(fp);
+	if (!mimeinfo_) {
+		fclose(fp);
+		return NULL;
+	}
+
+	if (!MSG_IS_ENCRYPTED(msginfo->flags) &&
+	    rfc2015_is_encrypted(mimeinfo_)) {
+		MSG_SET_TMP_FLAGS(msginfo->flags, MSG_ENCRYPTED);
+	}
+
+	if (MSG_IS_ENCRYPTED(msginfo->flags) &&
+	    !msginfo->plaintext_file &&
+	    !msginfo->decryption_failed) {
+		fpos = ftell(fp);
+		rfc2015_decrypt_message(msginfo, mimeinfo_, fp);
+		if (msginfo->plaintext_file &&
+		    !msginfo->decryption_failed) {
+			fclose(fp);
+			procmime_mimeinfo_free_all(mimeinfo_);
+			if ((fp = procmsg_open_message(msginfo)) == NULL)
+				return NULL;
+			mimeinfo_ = procmime_scan_mime_header(fp);
+			if (!mimeinfo_) {
+				fclose(fp);
+				return NULL;
+			}
+		} else {
+			if (fseek(fp, fpos, SEEK_SET) < 0)
+				perror("fseek");
+		}
+	}
+
+	if (mimeinfo) *mimeinfo = mimeinfo_;
+	return fp;
+}
+
 
 /*
  * plain contains an entire mime object.
@@ -752,7 +858,7 @@ gint rfc2015_encrypt(const gchar *file, GSList *recp_list,
 	}
 
 	/* Open the source file */
-	if ((fp = fopen(file, "rb")) == NULL) {
+	if ((fp = g_fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		goto failure;
 	}
@@ -849,7 +955,7 @@ gint rfc2015_encrypt(const gchar *file, GSList *recp_list,
 		FILE_OP_ERROR(file, "fclose");
 		goto failure;
 	}
-	if ((fp = fopen(file, "wb")) == NULL) {
+	if ((fp = g_fopen(file, "wb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		goto failure;
 	}
@@ -1062,7 +1168,7 @@ gint rfc2015_sign(const gchar *file, GSList *key_list)
 	boundary = generate_mime_boundary("Signature");
 
 	/* Open the source file */
-	if ((fp = fopen(file, "rb")) == NULL) {
+	if ((fp = g_fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		goto failure;
 	}
@@ -1142,7 +1248,7 @@ gint rfc2015_sign(const gchar *file, GSList *key_list)
 		FILE_OP_ERROR(file, "fclose");
 		goto failure;
 	}
-	if ((fp = fopen(file, "wb")) == NULL) {
+	if ((fp = g_fopen(file, "wb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		goto failure;
 	}
@@ -1268,7 +1374,7 @@ gint rfc2015_clearsign(const gchar *file, GSList *key_list)
 	ssize_t bytesRW = 0;
 	gchar *micalg = NULL;
 
-	if ((fp = fopen(file, "rb")) == NULL) {
+	if ((fp = g_fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		goto failure;
 	}
@@ -1304,7 +1410,7 @@ gint rfc2015_clearsign(const gchar *file, GSList *key_list)
 		fp = NULL;
 		goto failure;
 	}
-	if ((fp = fopen(file, "wb")) == NULL) {
+	if ((fp = g_fopen(file, "wb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		goto failure;
 	}
